import { Event } from "@codingame/monaco-vscode-api/vscode/vs/base/common/event";
import { Disposable, IDisposable } from "@codingame/monaco-vscode-api/vscode/vs/base/common/lifecycle";
import { ILanguageService } from "@codingame/monaco-vscode-api/vscode/vs/editor/common/languages/language.service";
import { ITextModel } from "@codingame/monaco-vscode-api/vscode/vs/editor/common/model";
import { IMenuService } from "@codingame/monaco-vscode-api/vscode/vs/platform/actions/common/actions.service";
import { ICommandService } from "@codingame/monaco-vscode-api/vscode/vs/platform/commands/common/commands.service";
import { IConfigurationService } from "@codingame/monaco-vscode-api/vscode/vs/platform/configuration/common/configuration.service";
import { IContextKeyService } from "@codingame/monaco-vscode-api/vscode/vs/platform/contextkey/common/contextkey.service";
import { IDialogService } from "@codingame/monaco-vscode-api/vscode/vs/platform/dialogs/common/dialogs.service";
import { IInstantiationService } from "@codingame/monaco-vscode-api/vscode/vs/platform/instantiation/common/instantiation";
import { IQuickInputService } from "@codingame/monaco-vscode-api/vscode/vs/platform/quickinput/common/quickInput.service";
import { IWorkspaceFolder } from "@codingame/monaco-vscode-api/vscode/vs/platform/workspace/common/workspace";
import { IAdapterDescriptor, IAdapterManager, IConfig, IConfigurationManager, IDebugAdapter, IDebugAdapterDescriptorFactory, IDebugAdapterFactory, IDebugSession, IGuessedDebugger } from "@codingame/monaco-vscode-api/vscode/vs/workbench/contrib/debug/common/debug";
import { Debugger } from "../common/debugger.js";
import { ITaskService } from "@codingame/monaco-vscode-api/vscode/vs/workbench/contrib/tasks/common/taskService.service";
import { IEditorService } from "@codingame/monaco-vscode-api/vscode/vs/workbench/services/editor/common/editorService.service";
import { IExtensionService } from "@codingame/monaco-vscode-api/vscode/vs/workbench/services/extensions/common/extensions.service";
import { ILifecycleService } from "@codingame/monaco-vscode-api/vscode/vs/workbench/services/lifecycle/common/lifecycle.service";
export interface IAdapterManagerDelegate {
    onDidNewSession: Event<IDebugSession>;
    configurationManager(): IConfigurationManager;
}
export declare class AdapterManager extends Disposable implements IAdapterManager {
    private readonly delegate;
    private readonly editorService;
    private readonly configurationService;
    private readonly quickInputService;
    private readonly instantiationService;
    private readonly commandService;
    private readonly extensionService;
    private readonly contextKeyService;
    private readonly languageService;
    private readonly dialogService;
    private readonly lifecycleService;
    private readonly tasksService;
    private readonly menuService;
    private debuggers;
    private adapterDescriptorFactories;
    private debugAdapterFactories;
    private debuggersAvailable;
    private debugExtensionsAvailable;
    private readonly _onDidRegisterDebugger;
    private readonly _onDidDebuggersExtPointRead;
    private breakpointContributions;
    private debuggerWhenKeys;
    private taskLabels;
    private earlyActivatedExtensions;
    private usedDebugTypes;
    constructor(delegate: IAdapterManagerDelegate, editorService: IEditorService, configurationService: IConfigurationService, quickInputService: IQuickInputService, instantiationService: IInstantiationService, commandService: ICommandService, extensionService: IExtensionService, contextKeyService: IContextKeyService, languageService: ILanguageService, dialogService: IDialogService, lifecycleService: ILifecycleService, tasksService: ITaskService, menuService: IMenuService);
    private registerListeners;
    private updateTaskLabels;
    private updateDebugAdapterSchema;
    registerDebugAdapterFactory(debugTypes: string[], debugAdapterLauncher: IDebugAdapterFactory): IDisposable;
    hasEnabledDebuggers(): boolean;
    createDebugAdapter(session: IDebugSession): IDebugAdapter | undefined;
    substituteVariables(debugType: string, folder: IWorkspaceFolder | undefined, config: IConfig): Promise<IConfig>;
    runInTerminal(debugType: string, args: DebugProtocol.RunInTerminalRequestArguments, sessionId: string): Promise<number | undefined>;
    registerDebugAdapterDescriptorFactory(debugAdapterProvider: IDebugAdapterDescriptorFactory): IDisposable;
    unregisterDebugAdapterDescriptorFactory(debugAdapterProvider: IDebugAdapterDescriptorFactory): void;
    getDebugAdapterDescriptor(session: IDebugSession): Promise<IAdapterDescriptor | undefined>;
    getDebuggerLabel(type: string): string | undefined;
    get onDidRegisterDebugger(): Event<void>;
    get onDidDebuggersExtPointRead(): Event<void>;
    canSetBreakpointsIn(model: ITextModel): boolean;
    getDebugger(type: string): Debugger | undefined;
    getEnabledDebugger(type: string): Debugger | undefined;
    someDebuggerInterestedInLanguage(languageId: string): boolean;
    guessDebugger(gettingConfigurations: boolean): Promise<IGuessedDebugger | undefined>;
    private initExtensionActivationsIfNeeded;
    activateDebuggers(activationEvent: string, debugType?: string): Promise<void>;
}
