import { CancellationToken } from "@codingame/monaco-vscode-api/vscode/vs/base/common/cancellation";
import { Event } from "@codingame/monaco-vscode-api/vscode/vs/base/common/event";
import { IDisposable } from "@codingame/monaco-vscode-api/vscode/vs/base/common/lifecycle";
import { URI as uri } from "@codingame/monaco-vscode-api/vscode/vs/base/common/uri";
import { IConfigurationService } from "@codingame/monaco-vscode-api/vscode/vs/platform/configuration/common/configuration.service";
import { IContextKeyService } from "@codingame/monaco-vscode-api/vscode/vs/platform/contextkey/common/contextkey.service";
import { IInstantiationService } from "@codingame/monaco-vscode-api/vscode/vs/platform/instantiation/common/instantiation";
import { ILogService } from "@codingame/monaco-vscode-api/vscode/vs/platform/log/common/log.service";
import { IQuickInputService } from "@codingame/monaco-vscode-api/vscode/vs/platform/quickinput/common/quickInput.service";
import { IStorageService } from "@codingame/monaco-vscode-api/vscode/vs/platform/storage/common/storage.service";
import { IUriIdentityService } from "@codingame/monaco-vscode-api/vscode/vs/platform/uriIdentity/common/uriIdentity.service";
import { IWorkspaceContextService } from "@codingame/monaco-vscode-api/vscode/vs/platform/workspace/common/workspace.service";
import { IExtensionService } from "@codingame/monaco-vscode-api/vscode/vs/workbench/services/extensions/common/extensions.service";
import { IHistoryService } from "@codingame/monaco-vscode-api/vscode/vs/workbench/services/history/common/history.service";
import { DebugConfigurationProviderTriggerKind, IAdapterManager, IConfig, IConfigPresentation, IConfigurationManager, IDebugConfigurationProvider, ILaunch } from "@codingame/monaco-vscode-api/vscode/vs/workbench/contrib/debug/common/debug";
interface IDynamicPickItem {
    label: string;
    launch: ILaunch;
    config: IConfig;
}
export declare class ConfigurationManager implements IConfigurationManager {
    private readonly adapterManager;
    private readonly contextService;
    private readonly configurationService;
    private readonly quickInputService;
    private readonly instantiationService;
    private readonly storageService;
    private readonly extensionService;
    private readonly historyService;
    private readonly uriIdentityService;
    private readonly logService;
    private launches;
    private selectedName;
    private selectedLaunch;
    private getSelectedConfig;
    private selectedType;
    private selectedDynamic;
    private toDispose;
    private readonly _onDidSelectConfigurationName;
    private configProviders;
    private debugConfigurationTypeContext;
    private readonly _onDidChangeConfigurationProviders;
    readonly onDidChangeConfigurationProviders: Event<void>;
    constructor(adapterManager: IAdapterManager, contextService: IWorkspaceContextService, configurationService: IConfigurationService, quickInputService: IQuickInputService, instantiationService: IInstantiationService, storageService: IStorageService, extensionService: IExtensionService, historyService: IHistoryService, uriIdentityService: IUriIdentityService, contextKeyService: IContextKeyService, logService: ILogService);
    registerDebugConfigurationProvider(debugConfigurationProvider: IDebugConfigurationProvider): IDisposable;
    unregisterDebugConfigurationProvider(debugConfigurationProvider: IDebugConfigurationProvider): void;
    hasDebugConfigurationProvider(debugType: string, triggerKind?: DebugConfigurationProviderTriggerKind): boolean;
    resolveConfigurationByProviders(folderUri: uri | undefined, type: string | undefined, config: IConfig, token: CancellationToken): Promise<IConfig | null | undefined>;
    resolveDebugConfigurationWithSubstitutedVariables(folderUri: uri | undefined, type: string | undefined, config: IConfig, token: CancellationToken): Promise<IConfig | null | undefined>;
    provideDebugConfigurations(folderUri: uri | undefined, type: string, token: CancellationToken): Promise<any[]>;
    getDynamicProviders(): Promise<{
        label: string;
        type: string;
        getProvider: () => Promise<IDebugConfigurationProvider | undefined>;
        pick: () => Promise<{
            launch: ILaunch;
            config: IConfig;
            label: string;
        } | undefined>;
    }[]>;
    getDynamicConfigurationsByType(type: string, token?: CancellationToken): Promise<IDynamicPickItem[]>;
    getAllConfigurations(): {
        launch: ILaunch;
        name: string;
        presentation?: IConfigPresentation;
    }[];
    removeRecentDynamicConfigurations(name: string, type: string): void;
    getRecentDynamicConfigurations(): {
        name: string;
        type: string;
    }[];
    private registerListeners;
    private initLaunches;
    private setCompoundSchemaValues;
    getLaunches(): ILaunch[];
    getLaunch(workspaceUri: uri | undefined): ILaunch | undefined;
    get selectedConfiguration(): {
        launch: ILaunch | undefined;
        name: string | undefined;
        getConfig: () => Promise<IConfig | undefined>;
        type: string | undefined;
    };
    get onDidSelectConfiguration(): Event<void>;
    getWorkspaceLaunch(): ILaunch | undefined;
    selectConfiguration(launch: ILaunch | undefined, name?: string, config?: IConfig, dynamicConfig?: {
        type?: string;
    }): Promise<void>;
    private setSelectedLaunchName;
    dispose(): void;
}
export {};
