import { CancellationToken } from "@codingame/monaco-vscode-api/vscode/vs/base/common/cancellation";
import { Event } from "@codingame/monaco-vscode-api/vscode/vs/base/common/event";
import { Disposable, IDisposable } from "@codingame/monaco-vscode-api/vscode/vs/base/common/lifecycle";
import { URI } from "@codingame/monaco-vscode-api/vscode/vs/base/common/uri";
import { IPosition, Position } from "@codingame/monaco-vscode-api/vscode/vs/editor/common/core/position";
import { IConfigurationService } from "@codingame/monaco-vscode-api/vscode/vs/platform/configuration/common/configuration.service";
import { IInstantiationService } from "@codingame/monaco-vscode-api/vscode/vs/platform/instantiation/common/instantiation";
import { ILogService } from "@codingame/monaco-vscode-api/vscode/vs/platform/log/common/log.service";
import { INotificationService } from "@codingame/monaco-vscode-api/vscode/vs/platform/notification/common/notification.service";
import { IProductService } from "@codingame/monaco-vscode-api/vscode/vs/platform/product/common/productService.service";
import { ICustomEndpointTelemetryService } from "@codingame/monaco-vscode-api/vscode/vs/platform/telemetry/common/telemetry.service";
import { ITelemetryService } from "@codingame/monaco-vscode-api/vscode/vs/platform/telemetry/common/telemetry.service";
import { IUriIdentityService } from "@codingame/monaco-vscode-api/vscode/vs/platform/uriIdentity/common/uriIdentity.service";
import { IWorkspaceFolder } from "@codingame/monaco-vscode-api/vscode/vs/platform/workspace/common/workspace";
import { IWorkspaceContextService } from "@codingame/monaco-vscode-api/vscode/vs/platform/workspace/common/workspace.service";
import { RawDebugSession } from "./rawDebugSession.js";
import { AdapterEndEvent, IBreakpoint, IConfig, IDataBreakpoint, IDataBreakpointInfoResponse, IDebugLocationReferenced, IDebugSession, IDebugSessionOptions, IDebugger, IExceptionBreakpoint, IExceptionInfo, IFunctionBreakpoint, IInstructionBreakpoint, IMemoryRegion, IRawModelUpdate, IRawStoppedDetails, IReplElement, IStackFrame, IThread, LoadedSourceEvent, State } from "@codingame/monaco-vscode-api/vscode/vs/workbench/contrib/debug/common/debug";
import { IDebugService } from "@codingame/monaco-vscode-api/vscode/vs/workbench/contrib/debug/common/debug.service";
import { DebugCompoundRoot } from "../common/debugCompoundRoot.js";
import { DebugModel, Thread } from "@codingame/monaco-vscode-88aa9a78-75f5-5aaa-89e9-bbdd580b3b5c-common/vscode/vs/workbench/contrib/debug/common/debugModel";
import { Source } from "@codingame/monaco-vscode-88aa9a78-75f5-5aaa-89e9-bbdd580b3b5c-common/vscode/vs/workbench/contrib/debug/common/debugSource";
import { INewReplElementData } from "@codingame/monaco-vscode-88aa9a78-75f5-5aaa-89e9-bbdd580b3b5c-common/vscode/vs/workbench/contrib/debug/common/replModel";
import { IWorkbenchEnvironmentService } from "@codingame/monaco-vscode-api/vscode/vs/workbench/services/environment/common/environmentService.service";
import { IHostService } from "@codingame/monaco-vscode-api/vscode/vs/workbench/services/host/browser/host.service";
import { ILifecycleService } from "@codingame/monaco-vscode-api/vscode/vs/workbench/services/lifecycle/common/lifecycle.service";
import { IPaneCompositePartService } from "@codingame/monaco-vscode-api/vscode/vs/workbench/services/panecomposite/browser/panecomposite.service";
import { ITestService } from "@codingame/monaco-vscode-api/vscode/vs/workbench/contrib/testing/common/testService.service";
import { ITestResultService } from "@codingame/monaco-vscode-api/vscode/vs/workbench/contrib/testing/common/testResultService.service";
import { LiveTestResult } from "@codingame/monaco-vscode-6845754f-e617-5ed9-8aaa-6ca3653a9532-common/vscode/vs/workbench/contrib/testing/common/testResult";
import { IAccessibilityService } from "@codingame/monaco-vscode-api/vscode/vs/platform/accessibility/common/accessibility.service";
export declare class DebugSession implements IDebugSession, IDisposable {
    private id;
    private _configuration;
    root: IWorkspaceFolder | undefined;
    private model;
    private readonly debugService;
    private readonly telemetryService;
    private readonly hostService;
    private readonly configurationService;
    private readonly paneCompositeService;
    private readonly workspaceContextService;
    private readonly productService;
    private readonly notificationService;
    private readonly uriIdentityService;
    private readonly instantiationService;
    private readonly customEndpointTelemetryService;
    private readonly workbenchEnvironmentService;
    private readonly logService;
    private readonly testService;
    private readonly accessibilityService;
    parentSession: IDebugSession | undefined;
    rememberedCapabilities?: DebugProtocol.Capabilities;
    private _subId;
    raw: RawDebugSession | undefined;
    private initialized;
    private _options;
    private sources;
    private threads;
    private threadIds;
    private cancellationMap;
    private readonly rawListeners;
    private readonly globalDisposables;
    private fetchThreadsScheduler;
    private passFocusScheduler;
    private lastContinuedThreadId;
    private repl;
    private stoppedDetails;
    private readonly statusQueue;
    readonly correlatedTestRun?: LiveTestResult;
    private didTerminateTestRun?;
    private readonly _onDidChangeState;
    private readonly _onDidEndAdapter;
    private readonly _onDidLoadedSource;
    private readonly _onDidCustomEvent;
    private readonly _onDidProgressStart;
    private readonly _onDidProgressUpdate;
    private readonly _onDidProgressEnd;
    private readonly _onDidInvalidMemory;
    private readonly _onDidChangeREPLElements;
    private _name;
    private readonly _onDidChangeName;
    private _waitToResume?;
    constructor(id: string, _configuration: {
        resolved: IConfig;
        unresolved: IConfig | undefined;
    }, root: IWorkspaceFolder | undefined, model: DebugModel, options: IDebugSessionOptions | undefined, debugService: IDebugService, telemetryService: ITelemetryService, hostService: IHostService, configurationService: IConfigurationService, paneCompositeService: IPaneCompositePartService, workspaceContextService: IWorkspaceContextService, productService: IProductService, notificationService: INotificationService, lifecycleService: ILifecycleService, uriIdentityService: IUriIdentityService, instantiationService: IInstantiationService, customEndpointTelemetryService: ICustomEndpointTelemetryService, workbenchEnvironmentService: IWorkbenchEnvironmentService, logService: ILogService, testService: ITestService, testResultService: ITestResultService, accessibilityService: IAccessibilityService);
    getId(): string;
    setSubId(subId: string | undefined): void;
    getMemory(memoryReference: string): IMemoryRegion;
    get subId(): string | undefined;
    get configuration(): IConfig;
    get unresolvedConfiguration(): IConfig | undefined;
    get lifecycleManagedByParent(): boolean;
    get compact(): boolean;
    get saveBeforeRestart(): boolean;
    get compoundRoot(): DebugCompoundRoot | undefined;
    get suppressDebugStatusbar(): boolean;
    get suppressDebugToolbar(): boolean;
    get suppressDebugView(): boolean;
    get autoExpandLazyVariables(): boolean;
    setConfiguration(configuration: {
        resolved: IConfig;
        unresolved: IConfig | undefined;
    }): void;
    getLabel(): string;
    setName(name: string): void;
    get name(): string;
    get state(): State;
    get capabilities(): DebugProtocol.Capabilities;
    get onDidChangeState(): Event<void>;
    get onDidEndAdapter(): Event<AdapterEndEvent | undefined>;
    get onDidChangeReplElements(): Event<IReplElement | undefined>;
    get onDidChangeName(): Event<string>;
    get onDidCustomEvent(): Event<DebugProtocol.Event>;
    get onDidLoadedSource(): Event<LoadedSourceEvent>;
    get onDidProgressStart(): Event<DebugProtocol.ProgressStartEvent>;
    get onDidProgressUpdate(): Event<DebugProtocol.ProgressUpdateEvent>;
    get onDidProgressEnd(): Event<DebugProtocol.ProgressEndEvent>;
    get onDidInvalidateMemory(): Event<DebugProtocol.MemoryEvent>;
    initialize(dbgr: IDebugger): Promise<void>;
    launchOrAttach(config: IConfig): Promise<void>;
    cancelCorrelatedTestRun(): void;
    terminate(restart?: boolean): Promise<void>;
    disconnect(restart?: boolean, suspend?: boolean): Promise<void>;
    restart(): Promise<void>;
    sendBreakpoints(modelUri: URI, breakpointsToSend: IBreakpoint[], sourceModified: boolean): Promise<void>;
    sendFunctionBreakpoints(fbpts: IFunctionBreakpoint[]): Promise<void>;
    sendExceptionBreakpoints(exbpts: IExceptionBreakpoint[]): Promise<void>;
    dataBytesBreakpointInfo(address: string, bytes: number): Promise<IDataBreakpointInfoResponse | undefined>;
    dataBreakpointInfo(name: string, variablesReference?: number): Promise<{
        dataId: string | null;
        description: string;
        canPersist?: boolean;
    } | undefined>;
    private _dataBreakpointInfo;
    sendDataBreakpoints(dataBreakpoints: IDataBreakpoint[]): Promise<void>;
    sendInstructionBreakpoints(instructionBreakpoints: IInstructionBreakpoint[]): Promise<void>;
    breakpointsLocations(uri: URI, lineNumber: number): Promise<IPosition[]>;
    getDebugProtocolBreakpoint(breakpointId: string): DebugProtocol.Breakpoint | undefined;
    customRequest(request: string, args: any): Promise<DebugProtocol.Response | undefined>;
    stackTrace(threadId: number, startFrame: number, levels: number, token: CancellationToken): Promise<DebugProtocol.StackTraceResponse | undefined>;
    exceptionInfo(threadId: number): Promise<IExceptionInfo | undefined>;
    scopes(frameId: number, threadId: number): Promise<DebugProtocol.ScopesResponse | undefined>;
    variables(variablesReference: number, threadId: number | undefined, filter: "indexed" | "named" | undefined, start: number | undefined, count: number | undefined): Promise<DebugProtocol.VariablesResponse | undefined>;
    evaluate(expression: string, frameId: number, context?: string, location?: {
        line: number;
        column: number;
        source: DebugProtocol.Source;
    }): Promise<DebugProtocol.EvaluateResponse | undefined>;
    restartFrame(frameId: number, threadId: number): Promise<void>;
    private setLastSteppingGranularity;
    next(threadId: number, granularity?: DebugProtocol.SteppingGranularity): Promise<void>;
    stepIn(threadId: number, targetId?: number, granularity?: DebugProtocol.SteppingGranularity): Promise<void>;
    stepOut(threadId: number, granularity?: DebugProtocol.SteppingGranularity): Promise<void>;
    stepBack(threadId: number, granularity?: DebugProtocol.SteppingGranularity): Promise<void>;
    continue(threadId: number): Promise<void>;
    reverseContinue(threadId: number): Promise<void>;
    pause(threadId: number): Promise<void>;
    terminateThreads(threadIds?: number[]): Promise<void>;
    setVariable(variablesReference: number, name: string, value: string): Promise<DebugProtocol.SetVariableResponse | undefined>;
    setExpression(frameId: number, expression: string, value: string): Promise<DebugProtocol.SetExpressionResponse | undefined>;
    gotoTargets(source: DebugProtocol.Source, line: number, column?: number): Promise<DebugProtocol.GotoTargetsResponse | undefined>;
    goto(threadId: number, targetId: number): Promise<DebugProtocol.GotoResponse | undefined>;
    loadSource(resource: URI): Promise<DebugProtocol.SourceResponse | undefined>;
    getLoadedSources(): Promise<Source[]>;
    completions(frameId: number | undefined, threadId: number, text: string, position: Position, token: CancellationToken): Promise<DebugProtocol.CompletionsResponse | undefined>;
    stepInTargets(frameId: number): Promise<{
        id: number;
        label: string;
    }[] | undefined>;
    cancel(progressId: string): Promise<DebugProtocol.CancelResponse | undefined>;
    disassemble(memoryReference: string, offset: number, instructionOffset: number, instructionCount: number): Promise<DebugProtocol.DisassembledInstruction[] | undefined>;
    readMemory(memoryReference: string, offset: number, count: number): Promise<DebugProtocol.ReadMemoryResponse | undefined>;
    writeMemory(memoryReference: string, offset: number, data: string, allowPartial?: boolean): Promise<DebugProtocol.WriteMemoryResponse | undefined>;
    resolveLocationReference(locationReference: number): Promise<IDebugLocationReferenced>;
    getThread(threadId: number): Thread | undefined;
    getAllThreads(): IThread[];
    clearThreads(removeThreads: boolean, reference?: number | undefined): void;
    getStoppedDetails(): IRawStoppedDetails | undefined;
    rawUpdate(data: IRawModelUpdate): void;
    private waitForTriggeredBreakpoints;
    private fetchThreads;
    initializeForTest(raw: RawDebugSession): void;
    private registerListeners;
    private handleStop;
    private enableDependentBreakpoints;
    private getBreakpointsAtPosition;
    private onDidExitAdapter;
    private shutdown;
    dispose(): void;
    getSourceForUri(uri: URI): Source | undefined;
    getSource(raw?: DebugProtocol.Source): Source;
    private getRawSource;
    private getNewCancellationToken;
    private cancelAllRequests;
    getReplElements(): IReplElement[];
    hasSeparateRepl(): boolean;
    removeReplExpressions(): void;
    addReplExpression(stackFrame: IStackFrame | undefined, expression: string): Promise<void>;
    appendToRepl(data: INewReplElementData, isImportant?: boolean): void;
}
export declare class ThreadStatusScheduler extends Disposable {
    private pendingCancellations;
    private readonly threadOps;
    run(threadIdsP: Promise<number[]>, operation: (threadId: number, ct: CancellationToken) => Promise<unknown>): Promise<void>;
    cancel(threadIds?: readonly number[]): void;
}
