import { CountBadge } from "@codingame/monaco-vscode-api/vscode/vs/base/browser/ui/countBadge/countBadge";
import { HighlightedLabel, IHighlight } from "@codingame/monaco-vscode-api/vscode/vs/base/browser/ui/highlightedlabel/highlightedLabel";
import { CachedListVirtualDelegate } from "@codingame/monaco-vscode-api/vscode/vs/base/browser/ui/list/list";
import { IListAccessibilityProvider } from "@codingame/monaco-vscode-api/vscode/vs/base/browser/ui/list/listWidget";
import { IAsyncDataSource, ITreeNode, ITreeRenderer } from "@codingame/monaco-vscode-api/vscode/vs/base/browser/ui/tree/tree";
import { FuzzyScore } from "@codingame/monaco-vscode-api/vscode/vs/base/common/filters";
import { Disposable, DisposableStore, IDisposable } from "@codingame/monaco-vscode-api/vscode/vs/base/common/lifecycle";
import { IConfigurationService } from "@codingame/monaco-vscode-api/vscode/vs/platform/configuration/common/configuration.service";
import { IContextViewService } from "@codingame/monaco-vscode-api/vscode/vs/platform/contextview/browser/contextView.service";
import { IHoverService } from "@codingame/monaco-vscode-api/vscode/vs/platform/hover/browser/hover.service";
import { IInstantiationService } from "@codingame/monaco-vscode-api/vscode/vs/platform/instantiation/common/instantiation";
import { ILabelService } from "@codingame/monaco-vscode-api/vscode/vs/platform/label/common/label.service";
import { IEditorService } from "@codingame/monaco-vscode-api/vscode/vs/workbench/services/editor/common/editorService.service";
import { IDebugSession, IExpression, IReplElement, IReplElementSource, IReplOptions } from "@codingame/monaco-vscode-api/vscode/vs/workbench/contrib/debug/common/debug";
import { IDebugService } from "@codingame/monaco-vscode-api/vscode/vs/workbench/contrib/debug/common/debug.service";
import { Variable } from "@codingame/monaco-vscode-88aa9a78-75f5-5aaa-89e9-bbdd580b3b5c-common/vscode/vs/workbench/contrib/debug/common/debugModel";
import { RawObjectReplElement, ReplEvaluationInput, ReplEvaluationResult, ReplGroup, ReplOutputElement, ReplVariableElement } from "@codingame/monaco-vscode-88aa9a78-75f5-5aaa-89e9-bbdd580b3b5c-common/vscode/vs/workbench/contrib/debug/common/replModel";
import { AbstractExpressionsRenderer, IExpressionTemplateData, IInputBoxOptions } from "@codingame/monaco-vscode-88aa9a78-75f5-5aaa-89e9-bbdd580b3b5c-common/vscode/vs/workbench/contrib/debug/browser/baseDebugView";
import { DebugExpressionRenderer } from "@codingame/monaco-vscode-88aa9a78-75f5-5aaa-89e9-bbdd580b3b5c-common/vscode/vs/workbench/contrib/debug/browser/debugExpressionRenderer";
interface IReplEvaluationInputTemplateData {
    label: HighlightedLabel;
}
interface IReplGroupTemplateData {
    label: HTMLElement;
    source: SourceWidget;
    elementDisposable?: IDisposable;
}
interface IReplEvaluationResultTemplateData {
    value: HTMLElement;
    elementStore: DisposableStore;
}
interface IOutputReplElementTemplateData {
    container: HTMLElement;
    count: CountBadge;
    countContainer: HTMLElement;
    value: HTMLElement;
    source: SourceWidget;
    getReplElementSource(): IReplElementSource | undefined;
    elementDisposable: DisposableStore;
}
interface IRawObjectReplTemplateData {
    container: HTMLElement;
    expression: HTMLElement;
    name: HTMLElement;
    value: HTMLElement;
    label: HighlightedLabel;
    elementStore: DisposableStore;
}
export declare class ReplEvaluationInputsRenderer implements ITreeRenderer<ReplEvaluationInput, FuzzyScore, IReplEvaluationInputTemplateData> {
    static readonly ID = "replEvaluationInput";
    get templateId(): string;
    renderTemplate(container: HTMLElement): IReplEvaluationInputTemplateData;
    renderElement(element: ITreeNode<ReplEvaluationInput, FuzzyScore>, index: number, templateData: IReplEvaluationInputTemplateData): void;
    disposeTemplate(templateData: IReplEvaluationInputTemplateData): void;
}
export declare class ReplGroupRenderer implements ITreeRenderer<ReplGroup, FuzzyScore, IReplGroupTemplateData> {
    private readonly expressionRenderer;
    private readonly instaService;
    static readonly ID = "replGroup";
    constructor(expressionRenderer: DebugExpressionRenderer, instaService: IInstantiationService);
    get templateId(): string;
    renderTemplate(container: HTMLElement): IReplGroupTemplateData;
    renderElement(element: ITreeNode<ReplGroup, FuzzyScore>, _index: number, templateData: IReplGroupTemplateData): void;
    disposeTemplate(templateData: IReplGroupTemplateData): void;
}
export declare class ReplEvaluationResultsRenderer implements ITreeRenderer<ReplEvaluationResult | Variable, FuzzyScore, IReplEvaluationResultTemplateData> {
    private readonly expressionRenderer;
    static readonly ID = "replEvaluationResult";
    get templateId(): string;
    constructor(expressionRenderer: DebugExpressionRenderer);
    renderTemplate(container: HTMLElement): IReplEvaluationResultTemplateData;
    renderElement(element: ITreeNode<ReplEvaluationResult | Variable, FuzzyScore>, index: number, templateData: IReplEvaluationResultTemplateData): void;
    disposeTemplate(templateData: IReplEvaluationResultTemplateData): void;
}
export declare class ReplOutputElementRenderer implements ITreeRenderer<ReplOutputElement, FuzzyScore, IOutputReplElementTemplateData> {
    private readonly expressionRenderer;
    private readonly instaService;
    static readonly ID = "outputReplElement";
    constructor(expressionRenderer: DebugExpressionRenderer, instaService: IInstantiationService);
    get templateId(): string;
    renderTemplate(container: HTMLElement): IOutputReplElementTemplateData;
    renderElement({ element }: ITreeNode<ReplOutputElement, FuzzyScore>, index: number, templateData: IOutputReplElementTemplateData): void;
    private setElementCount;
    disposeTemplate(templateData: IOutputReplElementTemplateData): void;
    disposeElement(_element: ITreeNode<ReplOutputElement, FuzzyScore>, _index: number, templateData: IOutputReplElementTemplateData): void;
}
export declare class ReplVariablesRenderer extends AbstractExpressionsRenderer<IExpression | ReplVariableElement> {
    private readonly expressionRenderer;
    static readonly ID = "replVariable";
    get templateId(): string;
    constructor(expressionRenderer: DebugExpressionRenderer, debugService: IDebugService, contextViewService: IContextViewService, hoverService: IHoverService);
    renderElement(node: ITreeNode<IExpression | ReplVariableElement, FuzzyScore>, _index: number, data: IExpressionTemplateData): void;
    protected renderExpression(expression: IExpression | ReplVariableElement, data: IExpressionTemplateData, highlights: IHighlight[]): void;
    protected getInputBoxOptions(expression: IExpression): IInputBoxOptions | undefined;
}
export declare class ReplRawObjectsRenderer implements ITreeRenderer<RawObjectReplElement, FuzzyScore, IRawObjectReplTemplateData> {
    private readonly expressionRenderer;
    static readonly ID = "rawObject";
    constructor(expressionRenderer: DebugExpressionRenderer);
    get templateId(): string;
    renderTemplate(container: HTMLElement): IRawObjectReplTemplateData;
    renderElement(node: ITreeNode<RawObjectReplElement, FuzzyScore>, index: number, templateData: IRawObjectReplTemplateData): void;
    disposeTemplate(templateData: IRawObjectReplTemplateData): void;
}
export declare class ReplDelegate extends CachedListVirtualDelegate<IReplElement> {
    private readonly configurationService;
    private readonly replOptions;
    constructor(configurationService: IConfigurationService, replOptions: IReplOptions);
    getHeight(element: IReplElement): number;
    protected estimateHeight(element: IReplElement, ignoreValueLength?: boolean): number;
    getTemplateId(element: IReplElement): string;
    hasDynamicHeight(element: IReplElement): boolean;
}
export declare class ReplDataSource implements IAsyncDataSource<IDebugSession, IReplElement> {
    hasChildren(element: IReplElement | IDebugSession): boolean;
    getChildren(element: IReplElement | IDebugSession): Promise<IReplElement[]>;
}
export declare class ReplAccessibilityProvider implements IListAccessibilityProvider<IReplElement> {
    getWidgetAriaLabel(): string;
    getAriaLabel(element: IReplElement): string;
}
declare class SourceWidget extends Disposable {
    private readonly hoverService;
    private readonly labelService;
    private readonly el;
    private source?;
    private hover?;
    constructor(container: HTMLElement, editorService: IEditorService, hoverService: IHoverService, labelService: ILabelService);
    setSource(source?: IReplElementSource): void;
}
export {};
