
import { __decorate, __param } from '@codingame/monaco-vscode-api/external/tslib/tslib.es6';
import { AccessibleViewType, AccessibleViewProviderId } from '@codingame/monaco-vscode-api/vscode/vs/platform/accessibility/browser/accessibleView';
import { ContextKeyExpr } from '@codingame/monaco-vscode-api/vscode/vs/platform/contextkey/common/contextkey';
import { Disposable } from '@codingame/monaco-vscode-api/vscode/vs/base/common/lifecycle';
import { AccessibilityVerbositySettingId } from '@codingame/monaco-vscode-api/vscode/vs/workbench/contrib/accessibility/browser/accessibilityConfiguration';
import { localize } from '@codingame/monaco-vscode-api/vscode/vs/nls';
import { ICommandService } from '@codingame/monaco-vscode-api/vscode/vs/platform/commands/common/commands.service';
import { IViewsService } from '@codingame/monaco-vscode-api/vscode/vs/workbench/services/views/common/viewsService.service';
import { AccessibilityHelpNLS } from '@codingame/monaco-vscode-api/vscode/vs/editor/common/standaloneStrings';
import { SidebarFocusContext, FocusedViewContext } from '@codingame/monaco-vscode-api/vscode/vs/workbench/common/contextkeys';
import { VARIABLES_VIEW_ID, WATCH_VIEW_ID, CALLSTACK_VIEW_ID, LOADED_SCRIPTS_VIEW_ID, BREAKPOINTS_VIEW_ID } from '@codingame/monaco-vscode-api/vscode/vs/workbench/contrib/debug/common/debug';

class RunAndDebugAccessibilityHelp {
    constructor() {
        this.priority = 120;
        this.name = 'runAndDebugHelp';
        this.when = ( ContextKeyExpr.or(( ContextKeyExpr.and(( ContextKeyExpr.equals('activeViewlet', 'workbench.view.debug')), SidebarFocusContext)), ( ContextKeyExpr.equals(FocusedViewContext.key, VARIABLES_VIEW_ID)), ( ContextKeyExpr.equals(FocusedViewContext.key, WATCH_VIEW_ID)), ( ContextKeyExpr.equals(FocusedViewContext.key, CALLSTACK_VIEW_ID)), ( ContextKeyExpr.equals(FocusedViewContext.key, LOADED_SCRIPTS_VIEW_ID)), ( ContextKeyExpr.equals(FocusedViewContext.key, BREAKPOINTS_VIEW_ID))));
        this.type = AccessibleViewType.Help;
    }
    getProvider(accessor) {
        return ( new RunAndDebugAccessibilityHelpProvider(accessor.get(ICommandService), accessor.get(IViewsService)));
    }
}
let RunAndDebugAccessibilityHelpProvider = class RunAndDebugAccessibilityHelpProvider extends Disposable {
    constructor(_commandService, _viewsService) {
        super();
        this._commandService = _commandService;
        this._viewsService = _viewsService;
        this.id = AccessibleViewProviderId.RunAndDebug;
        this.verbositySettingKey = AccessibilityVerbositySettingId.Debug;
        this.options = { type: AccessibleViewType.Help };
        this._focusedView = this._viewsService.getFocusedViewName();
    }
    onClose() {
        switch (this._focusedView) {
            case 'Watch':
                this._commandService.executeCommand('workbench.debug.action.focusWatchView');
                break;
            case 'Variables':
                this._commandService.executeCommand('workbench.debug.action.focusVariablesView');
                break;
            case 'Call Stack':
                this._commandService.executeCommand('workbench.debug.action.focusCallStackView');
                break;
            case 'Breakpoints':
                this._commandService.executeCommand('workbench.debug.action.focusBreakpointsView');
                break;
            default:
                this._commandService.executeCommand('workbench.view.debug');
        }
    }
    provideContent() {
        return [
            ( localize(
            5856,
            "The Show Run and Debug view command{0} will open the current view.",
            '<keybinding:workbench.view.debug>'
        )),
            ( localize(
            5857,
            "The Debug: Start Debugging command{0} will start a debug session.",
            '<keybinding:workbench.action.debug.start>'
        )),
            ( localize(
            5858,
            "Access debug output and evaluate expressions in the debug console, which can be focused with{0}.",
            '<keybinding:workbench.panel.repl.view.focus>'
        )),
            AccessibilityHelpNLS.setBreakpoint,
            AccessibilityHelpNLS.addToWatch,
            ( localize(5859, "Once debugging, the following commands will be available:")),
            ( localize(
            5860,
            "- Debug: Restart Debugging command{0} will restart the current debug session.",
            '<keybinding:workbench.action.debug.restart>'
        )),
            ( localize(
            5861,
            "- Debug: Stop Debugging command{0} will stop the current debugging session.",
            '<keybinding:workbench.action.debug.stop>'
        )),
            ( localize(
            5862,
            "- Debug: Continue command{0} will continue execution until the next breakpoint.",
            '<keybinding:workbench.action.debug.continue>'
        )),
            ( localize(
            5863,
            "- Debug: Step Into command{0} will step into the next function call.",
            '<keybinding:workbench.action.debug.stepInto>'
        )),
            ( localize(
            5864,
            "- Debug: Step Over command{0} will step over the current function call.",
            '<keybinding:workbench.action.debug.stepOver>'
        )),
            ( localize(
            5865,
            "- Debug: Step Out command{0} will step out of the current function call.",
            '<keybinding:workbench.action.debug.stepOut>'
        )),
            ( localize(
            5866,
            'The debug viewlet is comprised of several views that can be focused with the following commands or navigated to via tab then arrow keys:'
        )),
            ( localize(
            5867,
            "- Debug: Focus Breakpoints View command{0} will focus the breakpoints view.",
            '<keybinding:workbench.debug.action.focusBreakpointsView>'
        )),
            ( localize(
            5868,
            "- Debug: Focus Call Stack View command{0} will focus the call stack view.",
            '<keybinding:workbench.debug.action.focusCallStackView>'
        )),
            ( localize(
            5869,
            "- Debug: Focus Variables View command{0} will focus the variables view.",
            '<keybinding:workbench.debug.action.focusVariablesView>'
        )),
            ( localize(
            5870,
            "- Debug: Focus Watch View command{0} will focus the watch view.",
            '<keybinding:workbench.debug.action.focusWatchView>'
        )),
            ( localize(
            5871,
            "The setting {0} controls whether watch variable changes are announced.",
            'accessibility.debugWatchVariableAnnouncements'
        )),
        ].join('\n');
    }
};
RunAndDebugAccessibilityHelpProvider = ( __decorate([
    ( __param(0, ICommandService)),
    ( __param(1, IViewsService))
], RunAndDebugAccessibilityHelpProvider));

export { RunAndDebugAccessibilityHelp };
