import { IJSONSchema, IJSONSchemaMap, IJSONSchemaSnippet } from "@codingame/monaco-vscode-api/vscode/vs/base/common/jsonSchema";
import { IWorkspaceFolder } from "@codingame/monaco-vscode-api/vscode/vs/platform/workspace/common/workspace";
import { IConfig, IDebuggerContribution, IDebugAdapter, IDebugger, IDebugSession, IAdapterManager, IDebuggerMetadata } from "@codingame/monaco-vscode-api/vscode/vs/workbench/contrib/debug/common/debug";
import { IDebugService } from "@codingame/monaco-vscode-api/vscode/vs/workbench/contrib/debug/common/debug.service";
import { IConfigurationService } from "@codingame/monaco-vscode-api/vscode/vs/platform/configuration/common/configuration.service";
import { IConfigurationResolverService } from "@codingame/monaco-vscode-api/vscode/vs/workbench/services/configurationResolver/common/configurationResolver.service";
import { ITextResourcePropertiesService } from "@codingame/monaco-vscode-api/vscode/vs/editor/common/services/textResourceConfiguration.service";
import { IExtensionDescription } from "@codingame/monaco-vscode-api/vscode/vs/platform/extensions/common/extensions";
import { ITelemetryEndpoint } from "@codingame/monaco-vscode-api/vscode/vs/platform/telemetry/common/telemetry";
import { IWorkbenchEnvironmentService } from "@codingame/monaco-vscode-api/vscode/vs/workbench/services/environment/common/environmentService.service";
import { ContextKeyExpression } from "@codingame/monaco-vscode-api/vscode/vs/platform/contextkey/common/contextkey";
import { IContextKeyService } from "@codingame/monaco-vscode-api/vscode/vs/platform/contextkey/common/contextkey.service";
export declare class Debugger implements IDebugger, IDebuggerMetadata {
    private adapterManager;
    private readonly configurationService;
    private readonly resourcePropertiesService;
    private readonly configurationResolverService;
    private readonly environmentService;
    private readonly debugService;
    private readonly contextKeyService;
    private debuggerContribution;
    private mergedExtensionDescriptions;
    private mainExtensionDescription;
    private debuggerWhen;
    private debuggerHiddenWhen;
    constructor(adapterManager: IAdapterManager, dbgContribution: IDebuggerContribution, extensionDescription: IExtensionDescription, configurationService: IConfigurationService, resourcePropertiesService: ITextResourcePropertiesService, configurationResolverService: IConfigurationResolverService, environmentService: IWorkbenchEnvironmentService, debugService: IDebugService, contextKeyService: IContextKeyService);
    merge(otherDebuggerContribution: IDebuggerContribution, extensionDescription: IExtensionDescription): void;
    startDebugging(configuration: IConfig, parentSessionId: string): Promise<boolean>;
    createDebugAdapter(session: IDebugSession): Promise<IDebugAdapter>;
    substituteVariables(folder: IWorkspaceFolder | undefined, config: IConfig): Promise<IConfig>;
    runInTerminal(args: DebugProtocol.RunInTerminalRequestArguments, sessionId: string): Promise<number | undefined>;
    get label(): string;
    get type(): string;
    get variables(): {
        [key: string]: string;
    } | undefined;
    get configurationSnippets(): IJSONSchemaSnippet[] | undefined;
    get languages(): string[] | undefined;
    get when(): ContextKeyExpression | undefined;
    get hiddenWhen(): ContextKeyExpression | undefined;
    get enabled(): boolean;
    get isHiddenFromDropdown(): boolean;
    get strings(): any;
    interestedInLanguage(languageId: string): boolean;
    hasInitialConfiguration(): boolean;
    hasDynamicConfigurationProviders(): boolean;
    hasConfigurationProvider(): boolean;
    getInitialConfigurationContent(initialConfigs?: IConfig[]): Promise<string>;
    getMainExtensionDescriptor(): IExtensionDescription;
    getCustomTelemetryEndpoint(): ITelemetryEndpoint | undefined;
    getSchemaAttributes(definitions: IJSONSchemaMap): IJSONSchema[] | null;
}
